/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.UUID;
import pp4u_manage.ManageConfigure;
import pp4u_manage.ManageKeys;
import pp4u_manage.ManageListener;
import pp4u_manage.ManageNodes;
import pp4u_manage.ThreadNodes;
import pp4u_manage.ManageSearch;
import pp4u_manage.ManageTasks;

/**
 * メインロジック
 * @author tondol
 */
public class AppMain {

    static private InfoNode localhost = null;
    static private String nodeID = null;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        BoxMain.showBox();
    }

    public static void initialize() {
        localhost = InfoNode.getLocalhost();
        nodeID = UUID.randomUUID().toString();
        debug("O アドレス取得 <" + localhost + ">");
        debug("O ノードID取得 <" + nodeID + ">");
    }

    /**
     * 起動時の処理
     */
    public static void load() {
        ManageConfigure.getInstance().load();
        ManageNodes.getInstance().load();
        ManageKeys.getInstance().load();
        ManageTasks.getInstance().load();
        connect();
    }

    /**
     * 終了時の処理
     */
    public static void exit() {
        close();
        ManageConfigure.getInstance().dispose();
        ManageTasks.getInstance().dispose();
        ManageKeys.getInstance().dispose();
        ManageNodes.getInstance().dispose();
        System.exit(0);
    }

    /**
     * 各セッションを開始
     */
    public static void connect() {
        try {
            initialize();
            ManageKeys.getInstance().start();
            ManageNodes.getInstance().start();
            ManageListener.getInstance().start();
            BoxMain.getInstance().setOnline(true);
            BoxMain.getInstance().applyState();
        } catch (Exception ex) {
            close();
        }
    }

    /**
     * 各セッションを終了
     */
    public static void close() {
        ManageListener.getInstance().stop();
        ManageNodes.getInstance().stop();
        ManageKeys.getInstance().stop();
        BoxMain.getInstance().setOnline(false);
        BoxMain.getInstance().applyState();
    }

    /**
     * ファイルをスキャン
     */
    public static void scan() {
        ManageKeys.getInstance().start();
    }

    /**
     * ファイルを検索
     */
    public static void search(String word) {
        ManageSearch.getInstance().search(word);
    }

    /**
     * ファイルをダウンロード
     */
    public static void download(int index) {
        try {
            InfoKey key = null;
            key = ManageSearch.getInstance().getKey(index);
            TaskDownload task = new TaskDownload(key);
            ManageTasks.getInstance().add(task);
        } catch (Exception ex) {
        }
    }

    /**
     * タスクを中断
     */
    public static void cancel(int index) {
        try {
            TaskAbstract task = null;
            task = ManageTasks.getInstance().getTask(index);
            ManageTasks.getInstance().remove(task);
        } catch (Exception ex) {
        }
    }

    /**
     * ノードを追加
     */
    public static void addNodes(String nodes) {
        try {
            String[] array = nodes.split("\n");
            for (String str : array) {
                InfoNode node = new InfoNode(str);
                ThreadNodes.getInstance().add(node);
            }
        } catch (Exception ex) {
        }
    }

    /**
     * ノードを削除
     */
    public static void removeNodes(String nodes) {
        try {
            String[] array = nodes.split("\n");
            for (String str : array) {
                InfoNode node = new InfoNode(str);
                ThreadNodes.getInstance().remove(node);
            }
        } catch (Exception ex) {
        }
    }

    /**
     * ローカルホストの取得
     */
    public static InfoNode getLocalhost() {
        return localhost;
    }

    /**
     * ユニークIDの取得
     */
    public static String getNodeID() {
        return nodeID;
    }

    /**
     * デバッグ表示
     */
    public static void debug(String msg) {
        BoxMain.getInstance().putMessage(msg);
    }

    /**
     * スタックトレース
     */
    public static void debug(Exception ex) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        ex.printStackTrace(pw);
        BoxMain.getInstance().putMessage(sw.toString());
    }
}
