/*
 * MainView.java
 *
 * Created on 2008/05/29, 18:01
 */
package pp4u;

import pp4u_manage.ManageSearch;
import java.awt.event.KeyEvent;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import pp4u_manage.ManageNodes;
import pp4u_manage.ManageTasks;

/**
 * メインフレーム
 * @author  tondol
 */
public class BoxMain extends javax.swing.JFrame {

    static private BoxMain instance = null;
    private boolean stateOnline = false;
    private boolean stateScan = false;

    /** Creates new form MainView */
    private BoxMain() {
        try {
            String looks = null;
            looks = UIManager.getSystemLookAndFeelClassName();
            UIManager.setLookAndFeel(looks);
            SwingUtilities.updateComponentTreeUI(this);
        } catch (Exception e) {
        } finally {
            initComponents();
            applyState();
        }
    }

    /**
     * インスタンス化
     */
    public static BoxMain getInstance() {
        if (instance == null) {
            instance = new BoxMain();
        }
        return instance;
    }

    /**
     * フォームを表示
     */
    public static void showBox() {
        getInstance().setVisible(true);
    }

    /**
     * 接続中フラグを更新
     */
    public void setOnline(boolean bool) {
        stateOnline = bool;
    }

    /**
     * スキャン中フラグを更新
     */
    public void setScan(boolean bool) {
        stateScan = bool;
    }

    /**
     * フラグ状態をUIに適用
     */
    public void applyState() {
        java.awt.EventQueue.invokeLater(new Runnable() {
            // 有効無効を設定
            public void run() {
                if (stateOnline) {
                    connectMenuItem.setEnabled(false);
                    disconnectMenuItem.setEnabled(true);
                } else {
                    connectMenuItem.setEnabled(true);
                    disconnectMenuItem.setEnabled(false);
                }
                if (stateScan) {
                    scanMenuItem.setEnabled(false);
                } else {
                    scanMenuItem.setEnabled(true);
                }
            }
        });
    }

    /**
     * ログとステータスラベルにメッセージを書き出す
     */
    public void putMessage(final String msg) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            // ログとキャレットを更新
            public void run() {
                labelStatus.setText(msg);
                taOutput.append(msg + "\n");
                int caret = taOutput.getText().length();
                taOutput.setCaretPosition(caret);
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        panelSearchBox = new javax.swing.JPanel();
        labelSearchBox = new javax.swing.JLabel();
        txtSearchBox = new javax.swing.JTextField();
        btnSearchBox = new javax.swing.JButton();
        labelTaskStatus = new javax.swing.JLabel();
        panelMain = new javax.swing.JPanel();
        splitPane = new javax.swing.JSplitPane();
        tabPaneUpper = new javax.swing.JTabbedPane();
        tabbedPaneSearch = new javax.swing.JPanel();
        scrollSearch = new javax.swing.JScrollPane();
        tableSearch = new javax.swing.JTable();
        tabbedPaneNodes = new javax.swing.JPanel();
        scrollNodes = new javax.swing.JScrollPane();
        tableNodes = new javax.swing.JTable();
        tabPaneLower = new javax.swing.JTabbedPane();
        tabbedPaneTasks = new javax.swing.JPanel();
        scrollTasks = new javax.swing.JScrollPane();
        tableTasks = new javax.swing.JTable();
        tabbedPaneLog = new javax.swing.JPanel();
        scrollLog = new javax.swing.JScrollPane();
        taOutput = new javax.swing.JTextArea();
        btnClear = new javax.swing.JButton();
        panelStatus = new javax.swing.JPanel();
        labelStatus = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        scanMenuItem = new javax.swing.JMenuItem();
        quitMenuItem = new javax.swing.JMenuItem();
        networkMenu = new javax.swing.JMenu();
        connectMenuItem = new javax.swing.JMenuItem();
        disconnectMenuItem = new javax.swing.JMenuItem();
        jMenuItem1 = new javax.swing.JMenuItem();
        optionMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Pure P4U");
        setMinimumSize(new java.awt.Dimension(480, 360));
        setName("P4Uframe"); // NOI18N
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
            public void windowOpened(java.awt.event.WindowEvent evt) {
                formWindowOpened(evt);
            }
        });
        getContentPane().setLayout(new java.awt.GridBagLayout());

        panelSearchBox.setMinimumSize(new java.awt.Dimension(0, 0));
        panelSearchBox.setPreferredSize(new java.awt.Dimension(0, 38));
        panelSearchBox.setLayout(new java.awt.GridBagLayout());

        labelSearchBox.setText("検索単語");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelSearchBox.add(labelSearchBox, gridBagConstraints);

        txtSearchBox.setFont(txtSearchBox.getFont().deriveFont(txtSearchBox.getFont().getSize()+2f));
        txtSearchBox.setMargin(new java.awt.Insets(3, 3, 3, 3));
        txtSearchBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchAction(evt);
            }
        });
        txtSearchBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                slectTextAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 5);
        panelSearchBox.add(txtSearchBox, gridBagConstraints);

        btnSearchBox.setMnemonic('S');
        btnSearchBox.setText("検索(S)");
        btnSearchBox.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        btnSearchBox.setMargin(new java.awt.Insets(2, 7, 2, 7));
        btnSearchBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 5);
        panelSearchBox.add(btnSearchBox, gridBagConstraints);

        labelTaskStatus.setText("Upload:0 / Download:0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 5);
        panelSearchBox.add(labelTaskStatus, gridBagConstraints);
        labelTaskStatus.getAccessibleContext().setAccessibleName("");

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(panelSearchBox, gridBagConstraints);

        panelMain.setMinimumSize(new java.awt.Dimension(0, 0));
        panelMain.setPreferredSize(new java.awt.Dimension(0, 0));
        panelMain.setLayout(new java.awt.GridBagLayout());

        splitPane.setDividerLocation(150);
        splitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        splitPane.setMinimumSize(new java.awt.Dimension(0, 0));
        splitPane.setPreferredSize(new java.awt.Dimension(400, 300));

        tabPaneUpper.setMinimumSize(new java.awt.Dimension(0, 0));
        tabPaneUpper.setPreferredSize(new java.awt.Dimension(200, 150));

        tabbedPaneSearch.setFont(tabbedPaneSearch.getFont().deriveFont(tabbedPaneSearch.getFont().getSize()+1f));
        tabbedPaneSearch.setLayout(new java.awt.GridBagLayout());

        tableSearch.setFont(tableSearch.getFont().deriveFont(tableSearch.getFont().getSize()+1f));
        tableSearch.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tableSearch.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tableSearchMouseClicked(evt);
            }
        });
        scrollSearch.setViewportView(tableSearch);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        tabbedPaneSearch.add(scrollSearch, gridBagConstraints);

        tabPaneUpper.addTab(" 検索(A) ", tabbedPaneSearch);

        tabbedPaneNodes.setFont(tabbedPaneNodes.getFont().deriveFont(tabbedPaneNodes.getFont().getSize()+1f));
        tabbedPaneNodes.setLayout(new java.awt.GridBagLayout());

        scrollNodes.setViewportView(tableNodes);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        tabbedPaneNodes.add(scrollNodes, gridBagConstraints);

        tabPaneUpper.addTab(" ノード(O) ", tabbedPaneNodes);

        tabPaneUpper.setMnemonicAt(0, 'O');
        tabPaneUpper.setMnemonicAt(1, 'A');

        splitPane.setTopComponent(tabPaneUpper);
        tabPaneUpper.getAccessibleContext().setAccessibleName("");

        tabPaneLower.setMinimumSize(new java.awt.Dimension(0, 0));

        tabbedPaneTasks.setFont(tabbedPaneTasks.getFont().deriveFont(tabbedPaneTasks.getFont().getSize()+1f));
        tabbedPaneTasks.setLayout(new java.awt.GridBagLayout());

        tableTasks.setFont(tableTasks.getFont().deriveFont(tableTasks.getFont().getSize()+1f));
        tableSearch.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tableTasks.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                tableTasksKeyPressed(evt);
            }
        });
        scrollTasks.setViewportView(tableTasks);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        tabbedPaneTasks.add(scrollTasks, gridBagConstraints);

        tabPaneLower.addTab(" タスク(T) ", tabbedPaneTasks);

        tabbedPaneLog.setFont(tabbedPaneLog.getFont().deriveFont(tabbedPaneLog.getFont().getSize()+1f));
        tabbedPaneLog.setMinimumSize(new java.awt.Dimension(150, 63));
        tabbedPaneLog.setLayout(new java.awt.GridBagLayout());

        taOutput.setColumns(20);
        taOutput.setEditable(false);
        taOutput.setFont(taOutput.getFont());
        taOutput.setLineWrap(true);
        taOutput.setRows(5);
        scrollLog.setViewportView(taOutput);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        tabbedPaneLog.add(scrollLog, gridBagConstraints);

        btnClear.setMnemonic('C');
        btnClear.setText("クリア(C)");
        btnClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        tabbedPaneLog.add(btnClear, gridBagConstraints);

        tabPaneLower.addTab(" ログ(L) ", tabbedPaneLog);

        tabPaneLower.setMnemonicAt(0, 'T');
        tabPaneLower.setMnemonicAt(1, 'L');

        splitPane.setBottomComponent(tabPaneLower);
        tabPaneLower.getAccessibleContext().setAccessibleName("");

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        panelMain.add(splitPane, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        getContentPane().add(panelMain, gridBagConstraints);

        panelStatus.setFont(panelStatus.getFont().deriveFont(panelStatus.getFont().getSize()+1f));
        panelStatus.setMinimumSize(new java.awt.Dimension(0, 0));
        panelStatus.setPreferredSize(new java.awt.Dimension(0, 25));
        panelStatus.setLayout(new java.awt.GridBagLayout());

        labelStatus.setFont(labelStatus.getFont().deriveFont(labelStatus.getFont().getSize()+1f));
        labelStatus.setText("stand-by ready");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelStatus.add(labelStatus, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        getContentPane().add(panelStatus, gridBagConstraints);

        menuBar.setAlignmentY(0.5F);
        menuBar.setPreferredSize(new java.awt.Dimension(0, 25));

        fileMenu.setMnemonic('F');
        fileMenu.setText("ファイル(F)");

        scanMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        scanMenuItem.setMnemonic('S');
        scanMenuItem.setText("スキャン(S)");
        scanMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        fileMenu.add(scanMenuItem);

        quitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
        quitMenuItem.setMnemonic('X');
        quitMenuItem.setText("終了(X)");
        quitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        fileMenu.add(quitMenuItem);

        menuBar.add(fileMenu);

        networkMenu.setMnemonic('N');
        networkMenu.setText("ネットワーク(N)");

        connectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        connectMenuItem.setFont(connectMenuItem.getFont());
        connectMenuItem.setMnemonic('C');
        connectMenuItem.setText("接続(C)");
        connectMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        networkMenu.add(connectMenuItem);

        disconnectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        disconnectMenuItem.setFont(disconnectMenuItem.getFont());
        disconnectMenuItem.setMnemonic('D');
        disconnectMenuItem.setText("切断(D)");
        disconnectMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        networkMenu.add(disconnectMenuItem);

        jMenuItem1.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem1.setFont(jMenuItem1.getFont());
        jMenuItem1.setMnemonic('N');
        jMenuItem1.setText("ノード設定(N)");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        networkMenu.add(jMenuItem1);

        optionMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        optionMenuItem.setFont(optionMenuItem.getFont());
        optionMenuItem.setMnemonic('O');
        optionMenuItem.setText("設定(O)");
        optionMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        networkMenu.add(optionMenuItem);

        menuBar.add(networkMenu);

        helpMenu.setMnemonic('H');
        helpMenu.setText("ヘルプ(H)");

        aboutMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        aboutMenuItem.setMnemonic('A');
        aboutMenuItem.setText("Pure P4Uについて(A)");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-600)/2, (screenSize.height-480)/2, 600, 480);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * 基本的なアクション
     */
    private void menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuActionPerformed
        final String action = evt.getActionCommand();
        SwingWorker worker = new SwingWorker() {
            // アクションに応じてスレッド生成
            @Override
            protected Object doInBackground() {
                if (action.equals("スキャン(S)")) {
                    AppMain.scan();
                } else if (action.equals("終了(X)")) {
                    formWindowClosing(null);
                } else if (action.equals("接続(C)")) {
                    AppMain.connect();
                } else if (action.equals("切断(D)")) {
                    AppMain.close();
                } else if (action.equals("ノード設定(N)")) {
                    BoxNode.showBox();
                } else if (action.equals("設定(O)")) {
                    BoxConfigure.showBox();
                } else if (action.equals("Pure P4Uについて(A)")) {
                    BoxAbout.showBox();
                }
                return null;
            }
        };
        worker.execute();
}//GEN-LAST:event_menuActionPerformed

    /**
     * テキストボックスを選択
     */
    private void slectTextAction(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_slectTextAction
        javax.swing.JTextField textBox = (javax.swing.JTextField) evt.getSource();
        textBox.selectAll();
}//GEN-LAST:event_slectTextAction

    /**
     * ファイルを検索する
     */
    private void searchAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchAction
        final String word = txtSearchBox.getText();
        SwingWorker worker = new SwingWorker() {
            // 検索開始
            @Override
            protected Object doInBackground() {
                AppMain.search(word);
                return null;
            }
            // 検索タブをアクティブにする
            @Override
            protected void done() {
                tabPaneUpper.setSelectedIndex(0);
            }
        };
        worker.execute();
}//GEN-LAST:event_searchAction

    /**
     * ログをクリア
     */
    private void clearAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearAction
        taOutput.setText("");
}//GEN-LAST:event_clearAction

    /**
     * ウィンドウを閉じる
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // 確認ダイアログを表示
        int returned = JOptionPane.showConfirmDialog(
                this, "本当に終了しちゃいますよ？",
                "Pure P4U 確認", JOptionPane.YES_NO_OPTION);
        if (returned != JOptionPane.YES_OPTION) {
            return;
        }
        // ワーカー定義
        SwingWorker worker = new SwingWorker() {
            // 終了処理を呼び出す
            @Override
            protected Object doInBackground() {
                AppMain.exit();
                return null;
            }
            // フォームを廃棄する
            @Override
            protected void done() {
                BoxMain.getInstance().dispose();
            }
        };
        worker.execute();
    }//GEN-LAST:event_formWindowClosing

    /**
     * 選択したファイルをダブルクリックでダウンロード
     */
    private void tableSearchMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tableSearchMouseClicked
        // ダブルクリックを認識
        if (evt.getClickCount() != 2) {
            return;
        }
        // 選択した行を取得
        final int index = tableSearch.getSelectedRow();
        SwingWorker worker = new SwingWorker() {
            // ダウンロード処理を呼び出す
            @Override
            protected Object doInBackground() {
                AppMain.download(index);
                return null;
            }
        };
        if (index != -1) {
            worker.execute();
        }
    }//GEN-LAST:event_tableSearchMouseClicked

    /**
     * 選択したタスクをデリートキーで中断
     */
    private void tableTasksKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_tableTasksKeyPressed
        if (evt.getKeyCode() != KeyEvent.VK_DELETE) {
            return;
        }
        final int index = tableTasks.getSelectedRow();
        // ワーカー定義
        SwingWorker worker = new SwingWorker() {
            // キャンセル処理を呼び出す
            @Override
            protected Object doInBackground() {
                AppMain.cancel(index);
                return null;
            }
        };
        if (index != -1) {
            worker.execute();
        }
    }//GEN-LAST:event_tableTasksKeyPressed

    /**
     * 起動時の処理
     */
    private void formWindowOpened(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowOpened
        SwingWorker worker = new SwingWorker() {
            // ロード処理を呼び出す
            @Override
            protected Object doInBackground() {
                AppMain.load();
                return null;
            }
            // テーブルモデルを設定
            @Override
            protected void done() {
                tableTasks.setModel(ManageTasks.getInstance());
                tableNodes.setModel(ManageNodes.getInstance());
                tableSearch.setModel(ManageSearch.getInstance());
            }
        };
        worker.execute();
    }//GEN-LAST:event_formWindowOpened
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JButton btnClear;
    private javax.swing.JButton btnSearchBox;
    private javax.swing.JMenuItem connectMenuItem;
    private javax.swing.JMenuItem disconnectMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JLabel labelSearchBox;
    private javax.swing.JLabel labelStatus;
    private javax.swing.JLabel labelTaskStatus;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenu networkMenu;
    private javax.swing.JMenuItem optionMenuItem;
    private javax.swing.JPanel panelMain;
    private javax.swing.JPanel panelSearchBox;
    private javax.swing.JPanel panelStatus;
    private javax.swing.JMenuItem quitMenuItem;
    private javax.swing.JMenuItem scanMenuItem;
    private javax.swing.JScrollPane scrollLog;
    private javax.swing.JScrollPane scrollNodes;
    private javax.swing.JScrollPane scrollSearch;
    private javax.swing.JScrollPane scrollTasks;
    private javax.swing.JSplitPane splitPane;
    private javax.swing.JTextArea taOutput;
    private javax.swing.JTabbedPane tabPaneLower;
    private javax.swing.JTabbedPane tabPaneUpper;
    private javax.swing.JPanel tabbedPaneLog;
    private javax.swing.JPanel tabbedPaneNodes;
    private javax.swing.JPanel tabbedPaneSearch;
    private javax.swing.JPanel tabbedPaneTasks;
    private javax.swing.JTable tableNodes;
    private javax.swing.JTable tableSearch;
    private javax.swing.JTable tableTasks;
    private javax.swing.JTextField txtSearchBox;
    // End of variables declaration//GEN-END:variables
}
