/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.RandomAccessFile;
import java.net.Socket;
import pp4u_packet.Packet;
import pp4u_packet.PacketGet;
import pp4u_packet.PacketPing;
import pp4u_packet.PacketPong;
import pp4u_packet.PacketPut;

/**
 * 受信タスク
 * @author tondol
 */
public class TaskDownload extends TaskAbstract implements Runnable {

    private Socket socket = null;
    private ObjectInputStream in = null;
    private ObjectOutputStream out = null;
    private volatile InfoNode gaddr = null;
    private volatile InfoNode paddr = null;
    private volatile Thread thread = null;
    private volatile InfoKey key = null;
    private volatile long current = 0;
    private volatile long length = 0;

    public TaskDownload(InfoKey key) throws Exception {
        try {
            String hostname = key.getNode().getHostname();
            int port = key.getNode().getPort();
            this.socket = new Socket(hostname, port);
            this.out = new ObjectOutputStream(socket.getOutputStream());
            this.out.flush();
            PacketPing ping = new PacketPing(PacketPing.TRANSFER);
            this.out.writeObject(ping);
            this.in = new ObjectInputStream(socket.getInputStream());
            PacketPong pong = (PacketPong) in.readObject();

            String ghost = socket.getInetAddress().getHostAddress();
            int gport = pong.getAddress().getPort();
            this.paddr = pong.getAddress();
            this.gaddr = new InfoNode(ghost, gport);
            this.key = key;

            thread = new Thread(this);
            thread.start();
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    public void stop() {
        try {
            if (socket != null) {
                socket.close();
            }
        } catch (IOException iex) {
        }
    }

    public void run() {
        RandomAccessFile ost = null;

        try {
            // ファイル読み込み
            File file = new File(key.getDownloadPath());
            this.length = key.getSize();
            this.current = file.length();
            ost = new RandomAccessFile(file, "rw");
            ost.seek(current);
            // 終了条件
            while (length - current > 0) {
                // GETを送信してPUTを受信する
                PacketGet get = new PacketGet(key, current);
                send(get);
                PacketPut put = (PacketPut) recv();
                // ブロックを書き込む
                int loaded = (int) Math.min(length - current, 65536);
                ost.write(put.getBytes(), 0, loaded);
                current += loaded;
            }
        } catch (EOFException eex) {
            // 原因不明だが時々発生する
        } catch (Exception ex) {
            AppMain.debug("# 受信タスクエラー");
            AppMain.debug("@ " + ex);
        } finally {
            try {
                if (ost != null) {
                    ost.close();
                }
            } catch (IOException iex) {
            } finally {
                stop();
                socket = null;
                thread = null;
            }
        }
    }

    private void send(Packet packet) throws IOException {
        out.writeObject(packet);
        out.reset();
    }

    private Packet recv() throws IOException, ClassNotFoundException {
        return (Packet) in.readObject();
    }

    //////////////////////////////////////////////
    // 取得・設定メソッド
    //////////////////////////////////////////////
    @Override
    public boolean isAlive() {
        if (socket == null) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public String getType() {
        return "受信タスク";
    }

    @Override
    public String getFilename() {
        if (key == null) {
            return null;
        } else {
            return key.getFilename();
        }
    }

    @Override
    public InfoNode getNode() {
        if (key == null) {
            return null;
        } else {
            return key.getNode();
        }
    }

    @Override
    public String getStatus() {
        int per = (int) (100.0 * (double) current / (double) length);
        return per + "% @ " + current + "/" + length;
    }

    //////////////////////////////////////////////
    // インターフェイスの実装
    //////////////////////////////////////////////
    @Override
    public boolean equals(Object obj) {
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
        final TaskDownload other = (TaskDownload) obj;
        if (!this.key.equals(other.key)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return key.hashCode();
    }
}
