/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import pp4u_packet.Packet;
import pp4u_packet.PacketPing;
import pp4u_packet.PacketPong;
import pp4u_packet.PacketQuery;
import pp4u_packet.PacketFound;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.List;
import pp4u_manage.ManageKeys;
import pp4u_manage.ThreadLinks;
import pp4u_manage.ThreadNodes;
import pp4u_packet.PacketQuit;

/**
 * 検索リンク - TaskAbstractは継承してなかったりする
 * @author tondol
 */
public class TaskLink implements Runnable {

    private Socket socket = null;
    private ObjectInputStream in = null;
    private ObjectOutputStream out = null;
    private volatile int priority = -1;
    private volatile Thread thread = null;
    private volatile boolean behind = false; // NATを噛ましているかどうか
    private volatile InfoNode gaddr = null; // 接続先のグローバルアドレス
    private volatile InfoNode paddr = null; // 接続先のプライベートアドレス
    private volatile String nodeID = null; // 接続先のノードID
    private volatile String linkID = null; // リンクID

    /**
     * 接続側のリンクを生成
     */
    public TaskLink(InfoNode node) throws Exception {
        try {
            this.socket = new Socket(node.getHostname(), node.getPort());
            this.out = new ObjectOutputStream(socket.getOutputStream());
            this.out.flush();
            PacketPing ping = new PacketPing(PacketPing.SEARCH);
            send(ping);
            this.in = new ObjectInputStream(socket.getInputStream());

            // 代わりの接続先を追加
            PacketPong pong = (PacketPong) recv();
            ThreadNodes manager = ThreadNodes.getInstance();
            if (pong.getType() != PacketPong.OK) {
                InfoNode[] arrayNodes = pong.getNodes();
                for (InfoNode alternative : arrayNodes) {
                    manager.add(alternative);
                }
                throw new Exception("検索リンク数の限界");
            }

            String ghost = socket.getInetAddress().getHostAddress();
            int gport = pong.getAddress().getPort();
            this.paddr = pong.getAddress();
            this.gaddr = new InfoNode(ghost, gport);
            if (!paddr.getHostname().equals(gaddr.getHostname())) {
                this.behind = true;
            }

            this.nodeID = pong.getNodeID();
            this.linkID = ping.getLinkID();

            this.thread = new Thread(this);
            this.thread.start();
            this.priority = 0;
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    /**
     * 受信側のリンクを生成
     */
    public TaskLink(Socket socket, ObjectInputStream in, PacketPing ping) throws Exception {
        try {
            this.socket = socket;
            this.in = in;
            this.out = new ObjectOutputStream(socket.getOutputStream());
            this.out.flush();

            // 接続数オーバーならノードを紹介
            ThreadLinks manager = ThreadLinks.getInstance();
            if (manager.getSize() < 5) {
                PacketPong pong = new PacketPong(PacketPong.OK);
                out.writeObject(pong);
            } else {
                TaskLink[] arrayLinks = manager.getArray();
                InfoNode[] arrayNodes = new InfoNode[arrayLinks.length];
                for (int i = 0; i < arrayLinks.length; i++) {
                    arrayNodes[i] = arrayLinks[i].getGrobalAddr();
                }
                PacketPong pong = new PacketPong(PacketPong.NG, arrayNodes);
                out.writeObject(pong);
                throw new Exception("検索リンク数の限界");
            }

            String ghost = socket.getInetAddress().getHostAddress();
            int gport = ping.getAddress().getPort();
            this.paddr = ping.getAddress();
            this.gaddr = new InfoNode(ghost, gport);
            if (!paddr.getHostname().equals(gaddr.getHostname())) {
                this.behind = true;
            }
            this.nodeID = ping.getNodeID();
            this.linkID = ping.getLinkID();

            this.thread = new Thread(this);
            this.thread.start();
            this.priority = 0;
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    /**
     * 接続停止
     */
    public void stop() {
        if (socket != null) {
            try {
                send(new PacketQuit());
            } catch (IOException iex) {
            }
            try {
                socket.close();
            } catch (IOException iex) {
            }
        }
    }

    /**
     * 受信スレッド
     */
    public void run() {
        try {
            while (socket.isConnected()) {
                Packet packet = recv();
                switch (packet.getCommand()) {
                    case Packet.QUERY:
                        PacketQuery query = (PacketQuery) packet;
                        callQuery(query);
                        break;
                    case Packet.FOUND:
                        PacketFound found = (PacketFound) packet;
                        callFound(found);
                        break;
                    case Packet.QUIT:
                        stop();
                        break;
                }
            }
        } catch (Exception ex) {
        } finally {
            stop();
            thread = null;
            socket = null;
            priority = -1;
        }
    }

    /**
     * QUERY受信時の処理
     */
    private void callQuery(PacketQuery query) {
        ThreadLinks linkManager = ThreadLinks.getInstance();
        ManageKeys keyManager = ManageKeys.getInstance();
        String localID = AppMain.getNodeID();

        // 深さが5以上なら廃棄
        if (query.getPath().size() > 5) {
            return;
        }
        // すでに到達してたら廃棄
        if (query.getPath().contains(localID)) {
            return;
        }

        // 検索
        List<InfoKey> matchedList = keyManager.searchLocal(query.getWord());
        if (!matchedList.isEmpty()) {
            PacketFound found = new PacketFound(matchedList, query.getPath());
            try {
                found.pop();
                send(found);
            } catch (IOException iex) {
            }
        }
        // 中継する
        try {
            query.push(localID);
            linkManager.broadcast(query, this);
        } catch (IOException iex) {
        }
    }

    /**
     * FOUND受信時の処理
     */
    private void callFound(PacketFound found) {
        ThreadLinks linkManager = ThreadLinks.getInstance();
        ManageKeys keyManager = ManageKeys.getInstance();

        if (found.getPath().size() == 0) {
            // キー受信 - リンク追加は未実装
            for (InfoKey key : found.getKeys()) {
                key.setLocalKey(false);
                keyManager.add(key);
            }
        } else {
            // リンク切れをチェック
            String nextID = found.pop();
            TaskLink node = linkManager.search(nextID);
            if (node == null) {
                return;
            }
            // 中継する
            try {
                node.send(found);
            } catch (IOException iex) {
            }
        }
    }

    /**
     * このノードにパケットを送信
     */
    public void send(Packet packet) throws IOException {
        out.writeObject(packet);
        out.reset();
    }

    /**
     * このノードからパケットを受信
     */
    public Packet recv() throws IOException, ClassNotFoundException {
        return (Packet) in.readObject();
    }

    //////////////////////////////////////////////
    // 取得・設定メソッド
    //////////////////////////////////////////////
    /**
     * 優先度を取得
     */
    public int getPriority() {
        return priority;
    }

    /**
     * グローバルアドレスを返す
     */
    public InfoNode getGrobalAddr() {
        return gaddr;
    }

    /**
     * プライベートアドレスを返す
     */
    public InfoNode getPrivateAddr() {
        return paddr;
    }

    /**
     * ノードIDを返す
     */
    public String getNodeID() {
        return nodeID;
    }

    /**
     * リンクIDを返す
     */
    public String getLinkID() {
        return linkID;
    }

    /**
     * NATの後ろにあるかどうか
     */
    public boolean isBehind() {
        return behind;
    }
}
