/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import pp4u.AppMain;

/**
 * 設定管理クラス - スレッドセーフ
 * @author tondol
 */
public class ManageConfigure {

    static private ManageConfigure instance = null;
    private volatile File upload = null;
    private volatile File download = null;
    private volatile String dynamic;
    private volatile int port;

    /**
     * コンストラクタなんて飾りです
     */
    private ManageConfigure() {
    }

    /**
     * 唯一のインスタンスを返却
     */
    static synchronized public ManageConfigure getInstance() {
        if (instance == null) {
            instance = new ManageConfigure();
        }
        return instance;
    }

    /**
     * 設定ファイルの読み込み
     */
    public synchronized void load() {
        FileInputStream fst = null;
        Properties pr = null;

        try {
            fst = new FileInputStream("pp4u.properties");
            pr = new Properties();
            pr.load(fst);
            if (pr.containsKey("port")) {
                port = Integer.parseInt(pr.getProperty("port"));
            }
            if (pr.containsKey("dynamic")) {
                dynamic = pr.getProperty("dynamic");
            }
            if (pr.containsKey("upload")) {
                upload = new File(pr.getProperty("upload"));
            }
            if (pr.containsKey("download")) {
                download = new File(pr.getProperty("download"));
            }
            //AppMain.debug("O 設定読込成功");
        } catch (Exception ex) {
            AppMain.debug("X 設定読込失敗");
            AppMain.debug("@ " + ex);
        } finally {
            try {
                if (fst != null) {
                    fst.close();
                }
            } catch (IOException iex) {
            }
        }
    }

    /**
     * 設定ファイルに書き込み
     */
    public synchronized void dispose() {
        FileOutputStream fst = null;
        Properties pr = null;

        try {
            fst = new FileOutputStream("pp4u.properties");
            pr = new Properties();
            if (port != 0) {
                pr.setProperty("port", Integer.toString(port));
            }
            if (dynamic != null) {
                pr.setProperty("dynamic", dynamic);
            }
            if (upload != null) {
                pr.setProperty("upload", upload.getPath());
            }
            if (download != null) {
                pr.setProperty("download", download.getPath());
            }
            pr.store(fst, null);
            //AppMain.debug("O 設定書込成功");
        } catch (Exception ex) {
            AppMain.debug("X 設定書込失敗");
            AppMain.debug("@ " + ex);
        } finally {
            try {
                if (fst != null) {
                    fst.flush();
                    fst.close();
                }
            } catch (IOException iex) {
            }
        }
    }

    public int getPort() {
        return port;
    }

    public String getDynamic() {
        return dynamic;
    }

    public File getUpload() {
        return upload;
    }

    public File getDownload() {
        return download;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public void setDynamic(String ddns) {
        this.dynamic = ddns;
    }

    public void setUpload(File upload) {
        this.upload = upload;
    }

    public void setDownload(File download) {
        this.download = download;
    }
}
