/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import javax.swing.table.AbstractTableModel;
import pp4u.InfoNode;
import pp4u.TaskLink;

/**
 * ノードとリンクをまとめて管理
 * @author tondol
 */
public class ManageNodes extends AbstractTableModel {

    static private ManageNodes instance = null;
    private ThreadNodes threadNodes = null;
    private ThreadLinks threadLinks = null;

    /**
     * コンストラクタなんて飾りです
     */
    private ManageNodes() {
        threadNodes = ThreadNodes.getInstance();
        threadLinks = ThreadLinks.getInstance();
    }

    /**
     * インスタンスを取得
     */
    static synchronized public ManageNodes getInstance() {
        if (instance == null) {
            instance = new ManageNodes();
        }
        return instance;
    }

    /**
     * ノードリスト読み込み
     */
    public synchronized void load() {
        threadNodes.load();
    }

    /**
     * ノード管理停止
     */
    public synchronized void dispose() {
        threadNodes.dispose();
    }

    /**
     * ノード管理起動
     */
    public synchronized void start() {
        threadNodes.start();
        threadLinks.start();
    }

    /**
     * ノード管理停止
     */
    public synchronized void stop() {
        threadLinks.stop();
        threadNodes.stop();
    }

    //////////////////////////////////////////////
    // TableModelの実装もね
    //////////////////////////////////////////////
    /**
     * 外部からテーブルを更新
     */
    public void publish() {
        fireTableDataChanged();
    }

    @Override
    public int getRowCount() {
        int numNodes = threadNodes.getSize();
        int numLinks = threadLinks.getSize();
        return numNodes + numLinks;
    }

    @Override
    public int getColumnCount() {
        return 4;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        // サイズを取得
        Object obj = null;
        int numNodes = threadNodes.getSize();
        int numLinks = threadLinks.getSize();

        if (rowIndex < numNodes) {
            // 対象ノードを取得
            InfoNode[] array = threadNodes.getArray();
            if (rowIndex >= array.length) {
                return obj;
            }
            InfoNode node = array[rowIndex];
            // 値を設定
            switch (columnIndex) {
                case 0:
                    obj = "待機(´・ω・｀)";
                    break;
                case 1:
                    obj = "未接続";
                    break;
                case 2:
                    obj = node;
                    break;
                case 3:
                    obj = "未接続";
                    break;
            }
        } else {
            // 対象リンクを取得
            TaskLink[] array = threadLinks.getArray();
            if (rowIndex - numNodes >= array.length) {
                return obj;
            }
            TaskLink link = array[rowIndex - numNodes];
            // 値を設定
            switch (columnIndex) {
                case 0:
                    obj = "検索リンク(ﾟ∀ﾟ)";
                    break;
                case 1:
                    if (link.isBehind()) {
                        obj = "Raw";
                    } else {
                        obj = "NAT";
                    }
                    break;
                case 2:
                    obj = link.getGrobalAddr();
                    break;
                case 3:
                    obj = link.getPriority();
                    break;
            }
        }
        return obj;
    }
    
    @Override
    public String getColumnName(int column) {
        String str = null;
        switch (column) {
            case 0:
                str = "状態";
                break;
            case 1:
                str = "種別";
                break;
            case 2:
                str = "接続先";
                break;
            case 3:
                str = "優先度";
                break;
        }
        return str;
    }
}

