/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import pp4u.*;
import java.io.IOException;
import java.util.ArrayList;
import pp4u_packet.Packet;

/**
 * リンク管理クラス - スレッドセーフ
 * @author tondol
 */
public class ThreadLinks implements Runnable {

    static private ThreadLinks instance = null;
    private ArrayList<TaskLink> links = null;
    private volatile Thread thread = null;

    /**
     * リンクリスト初期化
     */
    private ThreadLinks() {
        links = new ArrayList<TaskLink>();
    }

    /**
     * インスタンスを取得
     */
    static synchronized public ThreadLinks getInstance() {
        if (instance == null) {
            instance = new ThreadLinks();
        }
        return instance;
    }

    /**
     * リンク管理起動
     */
    public synchronized void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        //AppMain.debug("O リンク管理起動");
        }
    }

    /**
     * リンク管理停止
     */
    public synchronized void stop() {
        if (thread != null) {
            thread.interrupt();
        }
    }

    /**
     * リンク管理スレッド
     */
    public void run() {
        try {
            while (!thread.isInterrupted()) {
                callbacks();
                Thread.sleep(1 * 1000);
            }
        } catch (Exception ex) {
        } finally {
            thread = null;
            // リンクをすべて切断
            TaskLink[] arrayLinks = getArray();
            for (TaskLink link : arrayLinks) {
                move(link);
            }
        //AppMain.debug("O リンク管理終了");
        }
    }

    /**
     * コールバックメソッド
     */
    private void callbacks() {
        // 切断済みリンクをノードリストに移動
        TaskLink[] arrayLinks = getArray();
        for (TaskLink link : arrayLinks) {
            if (link.getPriority() == -1) {
                move(link);
            }
        }
    }

    /**
     * ノードIDに合致するリンクを返す
     */
    public TaskLink search(String others) {
        TaskLink[] arrayLinks = getArray();
        for (TaskLink link : arrayLinks) {
            if (link.getNodeID().equals(others)) {
                return link;
            }
        }
        return null;
    }

    /**
     * すべての検索リンクに送信(exceptを除く)
     */
    public void broadcast(Packet packet, TaskLink except) throws IOException {
        TaskLink[] arrayLinks = getArray();
        for (TaskLink link : arrayLinks) {
            if (link.equals(except)) {
                continue;
            }
            link.send(packet);
        }
    }

    /**
     * リンクを配列として取得
     */
    public TaskLink[] getArray() {
        synchronized (links) {
            return links.toArray(new TaskLink[0]);
        }
    }

    /**
     * リンク数を取得
     */
    public int getSize() {
        synchronized (links) {
            return links.size();
        }
    }

    /**
     * リンクを切断してノードリストに移動
     */
    public void move(TaskLink link) {
        ThreadNodes manager = ThreadNodes.getInstance();
        remove(link);
        manager.add(link.getGrobalAddr());
    }

    /**
     * リンクを追加
     */
    public void add(TaskLink link) {
        // 重複チェック
        TaskLink matched = search(link.getNodeID());
        if (matched != null) {
            String s1 = link.getLinkID();
            String s2 = matched.getLinkID();
            if (s1.compareTo(s2) > 0) {
                remove(matched);
            } else {
                return;
            }
        }
        // 追加
        synchronized (links) {
            links.add(link);
        }
        ManageNodes.getInstance().publish();
        AppMain.debug("+ 検索リンク追加 <" + link.getGrobalAddr() + ">");
    }

    /**
     * リンクを削除
     */
    public void remove(TaskLink link) {
        link.stop();
        synchronized (links) {
            links.remove(link);
        }
        ManageNodes.getInstance().publish();
        AppMain.debug("- 検索リンク削除 <" + link.getGrobalAddr() + ">");
    }
}
