/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import pp4u.AppMain;
import pp4u.InfoNode;
import pp4u.TaskLink;

/**
 * ノード管理クラス - スレッドセーフ
 * @author tondol
 */
public class ThreadNodes implements Runnable {

    static private ThreadNodes instance = null;
    private ArrayList<InfoNode> nodes = null;
    private volatile Thread thread = null;

    /**
     * コンストラクタなんて飾りです
     */
    private ThreadNodes() {
        nodes = new ArrayList<InfoNode>();
    }

    /**
     * インスタンスを取得
     */
    static synchronized public ThreadNodes getInstance() {
        if (instance == null) {
            instance = new ThreadNodes();
        }
        return instance;
    }

    /**
     * ノードリスト読み込み
     */
    public synchronized void load() {
        FileReader fst = null;
        BufferedReader ist = null;

        try {
            // ファイルを開く
            fst = new FileReader("pp4unode.txt");
            ist = new BufferedReader(fst);
            String line = null;
            // 終端まで読み込む
            while ((line = ist.readLine()) != null) {
                InfoNode info = new InfoNode(line);
                add(info);
            }
        } catch (IOException iex) {
            AppMain.debug("X ノードリスト読込失敗");
            AppMain.debug("@ " + iex);
        } finally {
            try {
                if (ist != null) {
                    ist.close();
                }
            } catch (IOException iex) {
            }
        }
    }

    /**
     * ノード管理停止
     */
    public synchronized void dispose() {
        FileWriter fst = null;
        PrintWriter ost = null;

        try {
            // ファイルを開く
            fst = new FileWriter("pp4unode.txt");
            ost = new PrintWriter(fst);
            // ノードリストを書き込んで切断
            InfoNode[] arrayNodes = getArray();
            for (InfoNode node : arrayNodes) {
                String line = node.toString();
                ost.println(line);
            }
        } catch (Exception ex) {
            AppMain.debug("X ノードリスト書込失敗");
            AppMain.debug("@ " + ex);
        } finally {
            if (ost != null) {
                ost.flush();
                ost.close();
            }
        }
    }

    /**
     * ノード管理起動
     */
    public synchronized void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        }
    }

    /**
     * ノード管理停止
     */
    public synchronized void stop() {
        if (thread != null) {
            thread.interrupt();
        //AppMain.debug("O ノード管理起動");
        }
    }

    /**
     * ノード管理スレッド
     */
    public void run() {
        try {
            while (!thread.isInterrupted()) {
                callbacks();
                Thread.sleep(10 * 1000);
            }
        } catch (Exception ex) {
        } finally {
            thread = null;
        //AppMain.debug("O ノード管理終了");
        }
    }

    /**
     * コールバックメソッド
     */
    private void callbacks() {
        ThreadLinks manager = ThreadLinks.getInstance();
        InfoNode[] arrayNodes = getArray();
        // 5個になるまで接続試行
        for (InfoNode node : arrayNodes) {
            if (manager.getSize() >= 5) {
                break;
            }
            move(node);
        }
    }

    /**
     * ノードを配列として取得
     */
    public InfoNode[] getArray() {
        synchronized (nodes) {
            return nodes.toArray(new InfoNode[0]);
        }
    }

    /**
     * ノード数を取得
     */
    public int getSize() {
        synchronized (nodes) {
            return nodes.size();
        }
    }

    /**
     * ノードを接続してリンクリストに移動
     */
    public void move(InfoNode node) {
        ThreadLinks manager = ThreadLinks.getInstance();
        try {
            TaskLink link = new TaskLink(node);
            manager.add(link);
            remove(node);
        } catch (Exception ex) {
        }
    }

    /**
     * ノードを追加 - 重複を許可しない
     */
    public void add(InfoNode node) {
        synchronized (nodes) {
            if (nodes.contains(node)) {
                return;
            }
        }
        // 既に100個以上あったら
        while (getSize() >= 100) {
            remove(nodes.get(0));
        }
        // 追加
        synchronized (nodes) {
            nodes.add(node);
        }
        ManageNodes.getInstance().publish();
        AppMain.debug("+ ノード追加 <" + node + ">");
    }

    /**
     * ノードを削除
     */
    public void remove(InfoNode node) {
        synchronized (nodes) {
            nodes.remove(node);
        }
        ManageNodes.getInstance().publish();
        AppMain.debug("- ノード削除 <" + node + ">");
    }
}

